my $config_options = {
#	'data_storage_product' => {
#		'product_code' => 'YeOldeDatabase',
#		'is_memory_based' => 1,
#		'is_file_based' => 1,
#		'is_local_proc' => 1,
#		'is_network_svc' => 1,
#	},
#	'catalog_instance' => {
#		'file_path' => 'test',
#		'server_ip' => '127.0.0.1',
#		'server_domain' => 'localhost',
#		'server_port' => 1,
#	},
#	'catalog_link_instance' => {
#		'local_dsn' => 'test',
#		'login_name' => 'jane',
#		'login_pass' => 'pwd',
#	},
#	'catalog_instance_opt' => {
#		'delim_ident' => 1,
#	},
#	'catalog_link_instance_opt' => {
#		'dbi_driver' => 'YeOldeDBD',
#		'auto_commit' => 1,
#	},
};
